/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_CORE_OPERATOR_Resize_H_
#define AIDGE_CORE_OPERATOR_Resize_H_

#include <memory>
#include <string>
#include <vector>

#include "aidge/backend/OperatorImpl.hpp"
#include "aidge/graph/Node.hpp"
#include "aidge/operator/OperatorTensor.hpp"
#include "aidge/utils/Registrar.hpp"
#include "aidge/utils/Types.h"

namespace Aidge {

class Resize_Op : public OperatorTensor,
                  public Registrable<Resize_Op, std::string, std::shared_ptr<OperatorImpl>(const Resize_Op&)>{

public:
    static const std::string Type;

    Resize_Op()
        : OperatorTensor(Type,
            {InputCategory::Data,
                InputCategory::OptionalData,
                InputCategory::OptionalData,
                InputCategory::OptionalData},
            1) {}

    /**
     * @brief Copy-constructor. Copy the operator attributes and its output tensor(s),
     * but not its input tensors (the new operator has no input associated).
     * @param op Operator to copy.
     */

    Resize_Op(const Resize_Op& op)
        : OperatorTensor(op)
    {
        if (!op.backend().empty()) {
            SET_IMPL_MACRO(Resize_Op, *this, op.backend());
        }
        else {
            mImpl = nullptr;
        }
    }

    /**
     * @brief Clone the operator using its copy-constructor.
     * @see Operator::Resize_Op
     */
    std::shared_ptr<Operator> clone() const override {
        return std::make_shared<Resize_Op>(*this);
    }

    bool dimsForwarded() const override final;
    bool forwardDims(bool allowDataDependency = false) override final;

    void setBackend(const std::string& name, DeviceIdx_t device = 0) override final;

    static const std::vector<std::string> getInputsName(){
        //  roi, scales, sizes, even if considered as const parameters/input
        return {"data_input", "roi ", "scales", "sizes"};
    }
    static const std::vector<std::string> getOutputsName(){
        return {"data_output"};
    }
};

inline std::shared_ptr<Node> Resize(const std::string &name = "") {

    return std::make_shared<Node>(std::make_shared<Resize_Op>(), name);
}

}  // namespace Aidge


#endif /* AIDGE_CORE_OPERATOR_Resize_H_ */