/********************************************************************************
 * Copyright (c) 2024 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <cstdint>  // std::int64_t

#include <pybind11/pybind11.h>

#include "aidge/operator/Shape.hpp"
#include "aidge/operator/OperatorTensor.hpp"

namespace py = pybind11;
namespace Aidge {

void init_Shape(py::module& m) {
    py::class_<Shape_Op, std::shared_ptr<Shape_Op>, OperatorTensor>(m, "ShapeOp", py::multiple_inheritance())
        .def(py::init<const std::int64_t,
                      const std::int64_t>(),
                py::arg("start"),
                py::arg("end"))
        .def_static("get_inputs_name", &Shape_Op::getInputsName)
        .def_static("get_outputs_name", &Shape_Op::getOutputsName);

    declare_registrable<Shape_Op>(m, "ShapeOp");

    m.def("Shape", &Shape, py::arg("start") = 0, py::arg("end") = -1, py::arg("name") = "");
}

}  // namespace Aidge
