/********************************************************************************
 * Copyright (c) 2024 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <pybind11/pybind11.h>
#include <string>
#include <vector>

#include "aidge/data/Tensor.hpp"
#include "aidge/operator/Split.hpp"
#include "aidge/operator/OperatorTensor.hpp"

namespace py = pybind11;
namespace Aidge {

void init_Split(py::module& m) {
    py::class_<Split_Op, std::shared_ptr<Split_Op>, Attributes, OperatorTensor>(m, "SplitOp", py::multiple_inheritance())
        .def(py::init<DimSize_t, std::int8_t, std::vector<DimSize_t>&>(),
                py::arg("nb_outputs"),
                py::arg("axis"),
                py::arg("split"))
        .def_static("get_inputs_name", &Split_Op::getInputsName)
        .def_static("get_outputs_name", &Split_Op::getOutputsName)
        .def_static("attributes_name", &Split_Op::staticGetAttrsName);

    declare_registrable<Split_Op>(m, "SplitOp");

    m.def("Split", &Split, py::arg("nb_outputs"), py::arg("axis") = 0, py::arg("split") = std::vector<DimSize_t>(), py::arg("name") = "");
}

}  // namespace Aidge
