/********************************************************************************
 * Copyright (c) 2024 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <cstddef>  // std::size_t
#include <cstdint>  // std::int64_t
#include <string>
#include <vector>

#include "aidge/operator/Shape.hpp"
#include "aidge/data/Tensor.hpp"
#include "aidge/utils/ErrorHandling.hpp"
#include "aidge/utils/Types.h"

void Aidge::Shape_OpImpl::forward() {
    const Shape_Op& op = dynamic_cast<const Shape_Op&>(mOp);
    const auto start = op.start();
    const auto end = op.end();

    op.getOutput(0)->getImpl()->copyCast(std::next(op.getInput(0)->dims().data(),
                                                   start),
                                         DataType::UInt64,
                                         end - start + 1);
}

const std::string Aidge::Shape_Op::Type = "Shape";

bool Aidge::Shape_Op::forwardDims(bool /*allowDataDependency*/) {
    if (inputsAssociated()) {
        if (mAttributes->template getAttr<std::int64_t>("Start") < 0)
            mAttributes->template getAttr<std::int64_t>("Start") += static_cast<std::int64_t>(getInput(0)->nbDims());
        if (mAttributes->template getAttr<std::int64_t>("End") < 0)
            mAttributes->template getAttr<std::int64_t>("End") += static_cast<std::int64_t>(getInput(0)->nbDims());

        const auto start = mAttributes->template getAttr<std::int64_t>("Start");
        const auto end = mAttributes->template getAttr<std::int64_t>("End");
        const auto nbDims = static_cast<std::int64_t>(getInput(0)->nbDims());
        const DimSize_t roi = end - start + 1;

        AIDGE_ASSERT(start < nbDims && end < nbDims, "'Start' and 'End' must be < {}", nbDims);
        AIDGE_ASSERT(roi> 1, "Unvalid ROI for Shape");

        mOutputs[0]->resize({roi});
        return true;
    }

    return false;
}

void Aidge::Shape_Op::setBackend(const std::string& name, Aidge::DeviceIdx_t device) {
    if (Registrar<Shape_Op>::exists({name})) {
        SET_IMPL_MACRO(Shape_Op, *this, name);
    }
    else {
        mImpl = std::make_shared<Shape_OpImpl>(*this);
    }
    mOutputs[0]->setBackend(name, device);
}
