/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include "aidge/operator/Transpose.hpp"

#include <cstddef>    // std::size_t
#include <cstdint>    // std::int64_t
#include <memory>
#include <stdexcept>  // std::runtime_error
#include <string>
#include <vector>

#include "aidge/data/Tensor.hpp"
#include "aidge/utils/ErrorHandling.hpp"
#include "aidge/utils/Registrar.hpp"
#include "aidge/utils/Types.h"

void Aidge::TransposeImpl::forward() {
    const Transpose_Op& op = dynamic_cast<const Transpose_Op&>(mOp);
    op.getOutput(0)->copyTranspose(*(op.getInput(0)), op.outputDimsOrder());
}

const std::string Aidge::Transpose_Op::Type = "Transpose";

bool Aidge::Transpose_Op::forwardDims(bool /*allowDataDependency*/) {
    if (inputsAssociated()) {
        std::vector<DimSize_t> outputDims;
        for (std::size_t i = 0; i < outputDimsOrder().size(); ++i) {
            outputDims.push_back(getInput(0)->dims()[outputDimsOrder()[i]]);
        }
        mOutputs[0]->resize(outputDims);
        return true;
    }
    return false;
}

void Aidge::Transpose_Op::setBackend(const std::string& name, Aidge::DeviceIdx_t device) {
    if (Registrar<Transpose_Op>::exists({name})){
        SET_IMPL_MACRO(Transpose_Op, *this, name);
    }
    else {
        mImpl = std::make_shared<TransposeImpl>(*this);
    }
    mOutputs[0]->setBackend(name, device);
}
