/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include "aidge/utils/Attributes.hpp"

#include <cctype>  // std::isdigit, std::islower, std::isupper, std::tolower,
                   // std::toupper
#include <string>

std::string Aidge::Attributes::snakeToPascal(const std::string& snakeCase) {
    std::string result;
    bool to_upper = true; // Start with uppercase for PascalCase

    for (char ch : snakeCase) {
        if (ch == '_') {
            to_upper = true; // Next character should be uppercase
        } else {
            if (to_upper) {
                result += std::toupper(ch);
                to_upper = false; // Reset flag after making a character uppercase
            } else {
                result += ch;
            }
        }
    }
    return result;
}

std::string Aidge::Attributes::pascalToSnake(const std::string& pascalCase) {
    std::string result;

    for (char ch : pascalCase) {
        if (std::isupper(ch)) {
            if (!result.empty()) {
                result += '_';
            }
            result += std::tolower(ch);
        } else {
            result += ch;
        }
    }
    return result;
}

bool Aidge::Attributes::isPascalCase(const std::string& str) {
    if (str.empty() || !std::isupper(str[0])) {
        return false;
    }

    bool expectUpper = false;
    for (size_t i = 1; i < str.size(); ++i) {
        if (str[i] == '_') {
            return false;
        }
        if (std::isupper(str[i])) {
            if (!expectUpper) {
                return false;
            }
            expectUpper = false;
        } else if (std::islower(str[i]) || std::isdigit(str[i])) {
            expectUpper = true;
        } else {
            return false;
        }
    }
    return true;
}

bool Aidge::Attributes::isSnakeCase(const std::string& str) {
    if (str.empty()) {
        return false;
    }

    bool lastCharWasUnderscore = false;
    for (char ch : str) {
        if (ch == '_') {
            if (lastCharWasUnderscore) {
                return false;
            }
            lastCharWasUnderscore = true;
        } else if (!std::islower(ch) && !std::isdigit(ch)) {
            return false;
        } else {
            lastCharWasUnderscore = false;
        }
    }
    return true;
}
